/*************************************************************************/
/*************************************************************************/
/******                                                             ******/
/******  FILE UTILITIES                                             ******/
/******                                                             ******/
/*************************************************************************/
/*************************************************************************/

#include <stdio.h>
#include <errno.h>

/*-----------------------------------------------------------------------*/
/* <Exists> returns true if named file exists and has read permission.   */
/*-----------------------------------------------------------------------*/

int Exists( FName )
char *FName;
{
  FILE *CheckFile;
  int ExistsOut;

  CheckFile = fopen( FName, "r" );
  ExistsOut = ( CheckFile != NULL );
  if ( ExistsOut ) fclose( CheckFile );
  return( ExistsOut );
}

/*-------------------------------------------------------------------------*/
/* <remove> deletes a file.  ANSI but missing from AT&T.  UNIX only!       */
/*-------------------------------------------------------------------------*/

int remove( filename )
char *filename;
{
  char SysCmd[80];
#ifdef MSDOS
  sprintf( SysCmd, "del %s >nul", filename );
#else
  sprintf( SysCmd, "rm %s 2>/dev/null", filename );
#endif
  system( SysCmd );
  return( 0 );
}

/*-------------------------------------------------------------------------*/
/* <rename> renames a file.  ANSI but missing from AT&T.  UNIX only!       */
/*-------------------------------------------------------------------------*/

int rename( oldname, newname )
char *oldname, *newname;
{
  char SysCmd[160];

  remove( newname );
#ifdef MSDOS
  sprintf( SysCmd, "ren %s %s >nul", oldname, newname );
#else
  sprintf( SysCmd, "mv %s %s 2>/dev/null", oldname, newname );
#endif
  system( SysCmd );
  return( 0 );
}

/*-------------------------------------------------------------------------*/
/* <filesize> returns size in bytes of file.  ANSI but missing from AT&T.  */
/*-------------------------------------------------------------------------*/

long filesize( InStream )
FILE *InStream;
{
  long CurrPos, LastPos;
  int SeekResult;

  LastPos = 0;
  CurrPos = ftell( InStream );
  if ( CurrPos >= 0 )
  {
    SeekResult = fseek( InStream, 0, 2 );  /* from end */
    if ( ! SeekResult )
    {
      LastPos = ftell( InStream );
      SeekResult = fseek( InStream, CurrPos, 0 );  /* from start */
    }  
  }
  return( LastPos );
}

/*-------------------------------------------------------------------------*/
/* <strerror> returns name for error number.  ANSI but missing from AT&T.  */
/*-------------------------------------------------------------------------*/

extern char *sys_errlist[];
extern int sys_nerr;

char *strerror( errornum )
int errornum;
{
  static char strerror_out[60];

  if ( ! errornum )
    strcpy( strerror_out, "" );
  else
  if ( errornum > sys_nerr )
    sprintf( strerror_out, "Unknown error #%d", errornum );
  else
    strcpy( strerror_out, sys_errlist[ errornum ] );
  return( strerror_out );
}

/*-------------------------------------------------------------------------*/
/* <Chk_open> checks file pointer, if null, returns errno.                 */
/*-------------------------------------------------------------------------*/

int Chk_open( InStream )
FILE *InStream;
{
  int ErrNo;

  ErrNo = errno;
  return( ( InStream != NULL ) ? 0 : ErrNo );
}

/*-------------------------------------------------------------------------*/
/* <Chk_rw> checks ferror for given stream, if true returns errno.         */
/*-------------------------------------------------------------------------*/

int Chk_rw( InStream )
FILE *InStream;
{
  int ErrNo;

  ErrNo = errno;
  return( ferror( InStream ) ? ErrNo : 0 );
}

/*-------------------------------------------------------------------------*/
/* <Chk_close> checks expression, if true, returns errno.                  */
/*-------------------------------------------------------------------------*/

int Chk_close( InResult )
int InResult;
{
  int ErrNo;

  ErrNo = errno;
  return( InResult ? ErrNo : 0 );
}

/*-------------------------------------------------------------------------*/
/* <ChkErr> checks for non zero argument.  Returns true if _err set.       */
/* Zero argument exits.  <0 displays message.                              */
/* >0 displays error name with message appended, assumes stream error.     */
/*-------------------------------------------------------------------------*/

int _err = 0;   /* extern */

int ChkErr( Result, Msg )
int Result;
char *Msg;
{
  char ErrorName[60];

  if ( Result )
  {
    fflush( stdout );
    fprintf( stderr, "\007\007\007\n\n" );
    if ( Result > 0 )
    {
      if ( Result > sys_nerr )
        sprintf( ErrorName, "Unknown error #%d", Result );
      else
        strcpy( ErrorName, sys_errlist[ Result ] );
      fprintf( stderr, "%s error %s file.\n\n", ErrorName, Msg );
    }
    else
      fprintf( stderr, "%s.\n\n", Msg );
    _err = 1;
  }  
  return( _err );
}

/*-------------------------------------------------------------------------*/
/* <StdErr> checks for errors on stdin/stdout streams.                     */
/*-------------------------------------------------------------------------*/

int StdErr()
{
  ChkErr( ( ferror( stdin ) != 0 ), "Trouble reading stdin" );
  ChkErr( ( ferror( stdout ) != 0 ), "Trouble writing stdin" );
  return( _err );
}

/*-------------------------------------------------------------------------*/
/* <InvalidFileNameChar> checks if character valid in MS-DOS name.         */
/*-------------------------------------------------------------------------*/

int InvalidFileNameChar( InCh )
int InCh;
{
  return( ( ( InCh >= 0 ) && ( InCh <= 31 ) ) || ( InCh == '"' ) ||
#ifdef MSDOS
          ( InCh == '/' ) ||
#else
          ( InCh == '\\' ) ||
#endif
          ( InCh == '[' ) || ( InCh == ']' ) ||
          ( InCh == ':' ) || ( InCh == '|' ) || ( InCh == '<' ) ||
          ( InCh == '>' ) || ( InCh == '+' ) || ( InCh == '=' ) ||
          ( InCh == ';' ) || ( InCh == ',' ) || ( InCh >= 128 ) );
}
