/*************************************************************************/
/*************************************************************************/
/******                                                             ******/
/******  STRING UTILITIES                                           ******/
/******                                                             ******/
/*************************************************************************/
/*************************************************************************/

#include <stdio.h>
#include <ctype.h>
#include "aibmisc.h"

/*-----------------------------------------------------------------------*/
/* <c_s> converts a character to a string of length one.                 */
/*-----------------------------------------------------------------------*/

char *c_s( Ch )
int Ch;
{
  static char c_s_out[2];

  c_s_out[ 0 ] = Ch;
  c_s_out[ 1 ] = '\0';
  return( c_s_out );
}

/*-----------------------------------------------------------------------*/
/* <fgets_prep> removes end of line characters from a line.              */
/* Useful after a line is read using fgets.                              */
/*-----------------------------------------------------------------------*/

char *fgets_prep( InLn )
char *InLn;
{
  int InLen;

  InLen = strlen( InLn );
  if ( InLen > 0 )
    if ( ! isprint( InLn[ InLen - 1 ] ) )
      InLn[ InLen - 1 ] = '\0';
  if ( InLen > 1 )
    if ( ! isprint( InLn[ InLen - 2 ] ) )
      InLn[ InLen - 2 ] = '\0';
  return( InLn );
}

/*-----------------------------------------------------------------------*/
/* <strupr> converts all lower case characters in a string to upper.     */
/* Modeled after command available in Zortech C.
/*-----------------------------------------------------------------------*/

char *strupr( InStr )
char *InStr;
{
  char *StrPtr;

  StrPtr = InStr;
  while ( *StrPtr )
  {
    *StrPtr = toupper( *StrPtr );
    StrPtr++;
  }
  return( InStr );
}

/*-----------------------------------------------------------------------*/
/* <Copy> Copies a portion of source string to target string.            */
/* Also returns pointer to the string.                                   */
/*-----------------------------------------------------------------------*/

char *Copy( SrcStr, StrStart, StrSize, TgtStr )
char *SrcStr;
int StrStart;
int StrSize;
char *TgtStr;
{
  int EndPos;

  TgtStr = '\0';
  if ( ! *SrcStr ) return( TgtStr );
  if ( StrStart >= strlen( SrcStr ) ) return( TgtStr );
  EndPos = StrStart + StrSize - 1;
  if ( EndPos > strlen( SrcStr ) - 1 )
    EndPos = strlen( SrcStr ) - 1;
  memmove( TgtStr, &SrcStr[ StrStart ], EndPos - StrStart + 1 );
  TgtStr[ EndPos + 1 ] = '\0';
  return( TgtStr );
}

/*-----------------------------------------------------------------------*/
/* <Strip> removes all leading and trailing spaces from a string.        */
/* Also returns pointer to the string.                                   */
/*-----------------------------------------------------------------------*/

char *Strip( InStr )
char *InStr;
{
  int StartPos, EndPos, ScanPos;

  if ( ! *InStr ) return( InStr );
  StartPos = -1;  EndPos = -1;  ScanPos = 0;
  for ( ScanPos = 0;  InStr[ ScanPos ];  ScanPos++ )
    if ( InStr[ ScanPos ] != ' ' )
    {
      if ( StartPos  == -1 ) StartPos = ScanPos;
      EndPos = ScanPos;
    }
  if ( StartPos >= 0 )
    memmove( InStr, &InStr[ StartPos ], EndPos - StartPos + 1 );
  if ( ( StartPos == -1 ) && ( EndPos == -1 ) )
    InStr[ 0 ] = '\0';
  else
  if ( EndPos < strlen( InStr ) )
    InStr[ EndPos - StartPos + 1 ] = '\0';
  return( InStr );
}

/*-----------------------------------------------------------------------*/
/* <DeLead> removes all leading occurrences of a selected character.     */
/* Also returns pointer to the string.                                   */
/*-----------------------------------------------------------------------*/

char *DeLead( InStr, Ch )
char *InStr;
int Ch;
{
  int StartPos = 0;

  if ( ( ! *InStr ) || ( *InStr != Ch ) ) return( InStr );
  while ( InStr[ StartPos ] == Ch ) StartPos++;
  memmove( InStr, &InStr[ StartPos ], strlen( InStr ) - StartPos + 1 );
  return( InStr );
}

/*-----------------------------------------------------------------------*/
/* <DeTrail> removes all trailing occurrences of a selected character.   */
/* Also returns pointer to the string.                                   */
/*-----------------------------------------------------------------------*/

char *DeTrail( InStr, Ch )
char *InStr;
int Ch;
{
  int EndPos, CurrLen;

  CurrLen = strlen( InStr );  EndPos = CurrLen - 1;
  if ( ( ! *InStr ) || ( InStr[ EndPos ] != Ch ) )
    return( InStr );
  while ( InStr[ EndPos ] == Ch ) EndPos--;
  InStr[ EndPos + 1 ] = '\0';
  return( InStr );
}

/*-----------------------------------------------------------------------*/
/* <CJ> centers a string in a field, by padding with leading spaces.     */
/* For Center Justified report headings.  Truncates if string too long.  */
/*-----------------------------------------------------------------------*/

char *CJ( InStr, Wid )
char *InStr;
int Wid;
{
  int CurrLen, LWid;

  Strip( InStr );
  CurrLen = strlen( InStr );
  if ( Wid - CurrLen >= 2 )
  {
    LWid = ( Wid - CurrLen ) / 2;
    memmove( &InStr[ LWid ], InStr, CurrLen + 1 );
    memset( InStr, ' ', LWid );
  }
  else
  if ( CurrLen > Wid )
    InStr[ Wid - 1 ] = '\0';
  return( InStr );
}

/*-----------------------------------------------------------------------*/
/* <CenterCh> centers a string in a field by padding on both sides       */
/* with a specified character.  Truncates if string is too long.         */
/*-----------------------------------------------------------------------*/

char *CenterCh( InStr, Ch, Wid )
char *InStr;
int Ch, Wid;
{
  int CurrLen, LWid;

  Strip( InStr );
  CurrLen = strlen( InStr );
  if ( CurrLen < Wid )
  {
    LWid = ( Wid - CurrLen ) / 2;
    if ( LWid )
    {
      memmove( &InStr[ LWid ], InStr, CurrLen );
      memset( InStr, Ch, LWid );
    }
    memset( &InStr[ LWid + CurrLen ], Ch, Wid - CurrLen - LWid );
  }
  InStr[ Wid ] = '\0';
  return( InStr );
}

/*-----------------------------------------------------------------------*/
/* <Center> centers a string in a field by padding on both sides         */
/* with spaces.  Truncates if string is too long.                        */
/*-----------------------------------------------------------------------*/

char *Center( InStr, Wid )
char *InStr;
int Wid;
{
  return( CenterCh( InStr, ' ', Wid ) );
}

/*-----------------------------------------------------------------------*/
/* <PadCh> pads a string in a field by padding with a specified char.    */
/* Positive width right justifies, negative left justifies.              */
/* String is truncated if longer than field width.                       */
/*-----------------------------------------------------------------------*/

char *PadCh( InStr, Ch, Wid )
char *InStr;
int Ch, Wid;
{
  int CurrLen, AbsWid;

  if ( Wid < 0 )
    AbsWid = 0 - Wid;
  else
    AbsWid = Wid;
  CurrLen = strlen( InStr );
  if ( CurrLen >= AbsWid )
    InStr[ AbsWid ] = '\0';
  else
  if ( Wid > 0 )
  {
    memmove( &InStr[ AbsWid - CurrLen ], InStr, CurrLen + 1 );
    memset( InStr, Ch, AbsWid - CurrLen );
  }
  else
  {
    memset( &InStr[ CurrLen ], Ch, AbsWid - CurrLen );
    InStr[ AbsWid ] = '\0';
  }
  return( InStr );
}

/*-----------------------------------------------------------------------*/
/* <Pad> pads a string in a field by padding with spaces.                */
/* Positive width right justifies, negative left justifies.              */
/* String is truncated if longer than field width.                       */
/*-----------------------------------------------------------------------*/

char *Pad( InStr, Wid )
char *InStr;
int Wid;
{
  return( PadCh( InStr, ' ', Wid ) );
}

/*-----------------------------------------------------------------------*/
/* <Rep> returns a string of n repetitions of a selected character.      */
/*-----------------------------------------------------------------------*/

char *Rep( Ch, Wid )
char Ch;
int Wid;
{
  static char RepOut[256];

  if ( Wid < 0 ) Wid = 0;
  if ( Wid ) memset( RepOut, Ch, Wid );
  RepOut[ Wid ] = '\0';
  return( RepOut );
}

/*-----------------------------------------------------------------------*/
/* <Space> returns a string of a selected number of spaces.              */
/*-----------------------------------------------------------------------*/

char *Space( Wid )
int Wid;
{
  return( Rep( ' ', Wid ) );
}

/*-----------------------------------------------------------------------*/
/* <Purge> removes all copies of a specified character from a string.    */
/*-----------------------------------------------------------------------*/

char *Purge( InStr, Ch )
char *InStr;
int Ch;
{
  char *InPos, *OutPos;

  InPos = InStr;  OutPos = InStr;
  while ( *InPos )
  {
    if ( *InPos != Ch )
    {
      *OutPos = *InPos;
      OutPos++;
    }
    InPos++;
  }
  *OutPos = '\0';
  return( InStr );
}

/*-----------------------------------------------------------------------*/
/* <Delete> removes n characters starting at selected index in a string. */
/* Modeled after command available in Pascal.                            */
/*-----------------------------------------------------------------------*/

char *Delete( InStr, DelPos, DelSize )
char *InStr;
int DelPos, DelSize;
{
  int CurrLen;

  CurrLen = strlen( InStr );
  memmove( &InStr[ DelPos ], &InStr[ DelPos + DelSize ],
           CurrLen - DelSize + 1 );
  return( InStr );
}

/*-----------------------------------------------------------------------*/
/* <Insert> inserts a substring into another string at any index.        */
/* Modeled after command available in Pascal.                            */
/*-----------------------------------------------------------------------*/

char *Insert( SubStr, InStr, InsPos )
char *SubStr, *InStr;
int InsPos;
{
  int InLen, SubLen;

  InLen = strlen( InStr );  SubLen = strlen( SubStr );
  memmove( &InStr[ InsPos + SubLen ], &InStr[ InsPos ],
           InLen - InsPos + 1 );
  memcpy( &InStr[ InsPos ], SubStr, SubLen );
  return( InStr );
}

/*-----------------------------------------------------------------------*/
/* <Pos> returns index to start of substring in string.                  */
/* Modeled after command available in Pascal.                            */
/*-----------------------------------------------------------------------*/

int Pos( SubStr, InStr )
char *SubStr, *InStr;
{
  int Found, SubLen, InLen, SrchEnd, SrchPos;
  char CompStr[256];

  SubLen = strlen( SubStr );
  InLen = strlen( InStr );
  if ( ! SubLen ) return( 0 );
  Found = 0;
  if ( InLen >= SubLen )
  {
    SrchPos = 0;
    Found = 0;
    SrchEnd = InLen - SubLen;
    while ( ( ! Found ) && ( SrchPos <= SrchEnd ) )
    {
      strncpy( CompStr, &InStr[ SrchPos ], SubLen );
      CompStr[ SubLen ] = '\0';
      if ( ! strcmp( SubStr, CompStr ) )
        Found = 1;
      else
        SrchPos++;
    }
  }
  if ( ! Found )
    return( InLen );  /* not found returns past end */
  else
    return( SrchPos );
}

/*-----------------------------------------------------------------------*/
/* <strstr> returns pointer to start of substring in string.             */
/* ANSI but missing from AT&T.                                           */
/*-----------------------------------------------------------------------*/

char *strstr( InStr, SubStr )
char *InStr, *SubStr;
{
  int PosIdx;

  if ( ! *SubStr ) return( InStr );
  PosIdx = Pos( SubStr, InStr );
  if ( PosIdx < strlen( InStr ) )
    return( &InStr[ PosIdx ] );
  else
    return( NULL );
}
