/*************************************************************************/
/*************************************************************************/
/******                                                             ******/
/******  SCREEN CONTROL UTILITIES                                   ******/
/******                                                             ******/
/*************************************************************************/
/*************************************************************************/

/* This file implements some basic screen control functions
   which would have otherwise required curses. */

#include <stdio.h>

/*-----------------------------------------------------------------------*/
/* <vid_init> returns true if tput operates (TERM variable set).         */
/*-----------------------------------------------------------------------*/

static char vid_clear_seq[40] = "";   /* clear screen */
static char vid_civis_seq[40] = "";   /* cursor invisible */
static char vid_cnorm_seq[40] = "";   /* cursor normal */
static char vid_rev_seq[40] = "";     /* start reverse attribute */
static char vid_blink_seq[40] = "";   /* start blink attribute */
static char vid_bold_seq[40] = "";    /* start bright attribute */
static char vid_dim_seq[40] = "";     /* start dim attribute */
static char vid_ul_seq[40] = "";      /* start underline attribute */
static char vid_norm_seq[40] = "";    /* reset all attributes */

int vid_ok = 0;                /* flags successful init */

/* <vid_read_tput> reads from tput, used by vid_init. */

int vid_read_tput( vid_seq, vid_cmd )
char *vid_seq, *vid_cmd;
{
  FILE *vid_pipe;
  int vid_read_size;
  char vid_cmd_out[50];

  *vid_seq = '\0';
  sprintf( vid_cmd_out, "tput %s 2>/dev/null", vid_cmd );
  vid_pipe = popen( vid_cmd_out, "r" );
  if ( vid_pipe != NULL )
  {
    vid_read_size = fread( vid_seq, 1, 40, vid_pipe );
    if ( vid_read_size > 0 )
      vid_seq[ vid_read_size ] = '\0';
    else
      vid_seq[ 0 ] = '\0';
    pclose( vid_pipe );
  }
  return( strlen( vid_seq ) );
}

int vid_init()
{
  int vid_init_out = 0;

  vid_init_out = vid_read_tput( vid_clear_seq, "clear" ) || vid_init_out;
  vid_init_out = vid_read_tput( vid_civis_seq, "civis" ) || vid_init_out;
  vid_init_out = vid_read_tput( vid_cnorm_seq, "cnorm" ) || vid_init_out;
  vid_init_out = vid_read_tput( vid_rev_seq, "rev" ) || vid_init_out;
  vid_init_out = vid_read_tput( vid_blink_seq, "blink" ) || vid_init_out;
  vid_init_out = vid_read_tput( vid_bold_seq, "bold" ) || vid_init_out;
  vid_init_out = vid_read_tput( vid_dim_seq, "dim" ) || vid_init_out;
  vid_init_out = vid_read_tput( vid_ul_seq, "smul" ) || vid_init_out;
  vid_init_out = vid_read_tput( vid_norm_seq, "sgr0" ) || vid_init_out;
  vid_ok = vid_init_out == 1;
  return( vid_init_out );
}
  
/*-----------------------------------------------------------------------*/
/* <vid_clear> clears the screen.                                        */
/*-----------------------------------------------------------------------*/

int vid_clear()
{
  printf( vid_clear_seq );
  fflush( stdout );
  return( 0 );
}

/*-----------------------------------------------------------------------*/
/* <vid_gotoXY> moves the cursor to a selected location.                 */
/*-----------------------------------------------------------------------*/

int vid_gotoXY( vid_x, vid_y )
int vid_x, vid_y;
{
  char vid_cmd[50];

  fflush( stdout );
  sprintf( vid_cmd, "tput cup %d %d 2>/dev/null", vid_y, vid_x );
  return( system( vid_cmd ) );
}

/*-----------------------------------------------------------------------*/
/* <vid_attr> sets selected video attributes on terminal.                */
/*-----------------------------------------------------------------------*/

#define v_norm 0
#define v_rev 1
#define v_blink 2
#define v_bold 4
#define v_dim 8
#define v_ul 16

int vid_attr( vid_attr_val )
int vid_attr_val;
{
  char vid_attr_seq[160];

  strcpy( vid_attr_seq, vid_norm_seq );
  if ( vid_attr_val & v_rev ) strcat( vid_attr_seq, vid_rev_seq );
  if ( vid_attr_val & v_blink ) strcat( vid_attr_seq, vid_blink_seq );
  if ( vid_attr_val & v_bold ) strcat( vid_attr_seq, vid_bold_seq );
  if ( vid_attr_val & v_dim ) strcat( vid_attr_seq, vid_dim_seq );
  if ( vid_attr_val & v_ul ) strcat( vid_attr_seq, vid_ul_seq );
  printf( vid_attr_seq );
  fflush( stdout );
  return( 0 );
}

/*-----------------------------------------------------------------------*/
/* <vid_cursor> sets selected cursor mode.                               */
/*-----------------------------------------------------------------------*/

#define v_con 0
#define v_coff 1

int vid_cursor( vid_cstyle )
int vid_cstyle;
{
  if ( vid_cstyle == v_coff )
    printf( vid_civis_seq );
  else
    printf( vid_cnorm_seq );
  fflush( stdout );
  return( 0 );
}

/*-----------------------------------------------------------------------*/
/* <vid_print> prints text, including attributes, at a location.         */
/*-----------------------------------------------------------------------*/

int vid_print( vid_x, vid_y, vid_attr_in, vid_txt )
int vid_x, vid_y, vid_attr_in;
char *vid_txt;
{
  vid_gotoXY( vid_x, vid_y );
  vid_attr( vid_attr_in );
  printf( vid_txt );
  fflush( stdout );
  return( 0 ); 
}
