// dtutil.c - Ashley Poole / Andrew Bedno - 1997.01.17
// Date and Time utilities.

char DayName[ 7 ][ 20 ] =
      { "Sunday", "Monday", "Tuesday", "Wednesday",
        "Thursday", "Friday", "Saturday" };

char MonthName[ 12 ][ 20 ] =
      { "January", "February", "March", "April", "May", "June", "July",
        "August", "September", "October", "November", "December" };
                   
int DaysPerMonth[ 12 ] =
      { 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 };

#define DATE_ITEMS 3               // Items in a date int array.
#define YEAR_ITEM 0                // position of YEAR in an array
#define MONTH_ITEM 1               // position of MONTH in an array
#define DAY_ITEM 2                 // position of DATE in an array

#define TIME_ITEMS 3               // Items in a time int array.
#define HOUR_ITEM 0                // position of HOURS in an array
#define MINUTE_ITEM 1              // position of MINUTES in an array
#define SECOND_ITEM 2              // position of SECONDS in an array

//  Put current time in to HMS int array.
void get_current_time(int current_time[TIME_ITEMS])
{
  time_t current_serial_time;   // create a time_t named current_serial_time
  struct tm tm_rec;             // set up a tm structure named tm_rec
  time(&current_serial_time);   // get time
  memcpy(&tm_rec, localtime(&current_serial_time), sizeof( tm_rec));
        // Add GMT support later !!!!!
  current_time[HOUR_ITEM]=tm_rec.tm_hour;       // Set hours -- 00 to 23
  current_time[MINUTE_ITEM]=tm_rec.tm_min;      // Set minutes -- 00 to 59
  current_time[SECOND_ITEM]=tm_rec.tm_sec;      // Set seconds -- 00 to 59
}


//  Put current date in to YMD int array.
void get_current_date(int current_date[DATE_ITEMS])
{
  time_t current_serial_time;   // create a time_t named current_serial_time
  struct tm tm_rec;             // set up a tm structure named tm_rec
  time(&current_serial_time);   // get time
  memcpy(&tm_rec, localtime(&current_serial_time), sizeof( tm_rec));
        // Add GMT support later !!!!!
  current_date[YEAR_ITEM]=((tm_rec.tm_year)+1900); // Base value 00 == 1900
  current_date[MONTH_ITEM]=((tm_rec.tm_mon)+1);  // Since 0=jan in this struct
  current_date[DAY_ITEM]=tm_rec.tm_mday;         // Between 1 and 31
}


// Return true if year/month/day makes valid date.
int ValidYMD( int InDate[DATE_ITEMS] )
{
  // Check for obvious out of range.
  if ( ( InDate[YEAR_ITEM] < 1970 ) || ( InDate[YEAR_ITEM] > 2069 ) ||
       ( InDate[MONTH_ITEM] < 1 ) || ( InDate[MONTH_ITEM] > 12 ) ||
       ( InDate[DAY_ITEM] < 1 ) || ( InDate[DAY_ITEM] > 31 ) )
    return( FALSE );
  // If leap year day, check if valid for year.
  if ( (InDate[MONTH_ITEM] == 2) && (InDate[DAY_ITEM] == 29) )
    if ( ((InDate[YEAR_ITEM] % 400) == 0) ||
         ( ((InDate[YEAR_ITEM] % 4) == 0) &&
           ((InDate[YEAR_ITEM] % 100) != 0) ) )
      return( TRUE );
    else
      return( FALSE );
  // Check for valid day in month.
  if ( InDate[DAY_ITEM] > DaysPerMonth[ InDate[MONTH_ITEM] - 1 ] )
    return( FALSE );
  return( TRUE );
}


// Returns true if hours/minutes/seconds make valid time.
int ValidHMS( int InTime[TIME_ITEMS] )
{
  // Check for range.
  return( ( InTime[HOUR_ITEM] >= 0 ) && ( InTime[HOUR_ITEM] <= 23 ) &&
          ( InTime[MINUTE_ITEM] >= 0 ) && ( InTime[MINUTE_ITEM] <= 59 ) &&
          ( InTime[SECOND_ITEM] >= 0 ) && ( InTime[SECOND_ITEM] <= 59 ) );
}


// Returns hours midnt 1997.01.01 given date/time int arrays, or -1 on error.
long DTHour( int InDate[DATE_ITEMS], int InTime[TIME_ITEMS] )
{
  long DaTotal = 0;
  int DTLp;

  if ( ! ValidYMD( InDate ) )
    return( -1.0 );
  if ( ! ValidHMS( InTime ) )
    return( -1.0 );
  // Loop to add leap year days.
  DTLp = 1997;
  while (DTLp < InDate[YEAR_ITEM])
  {
    if ( ((DTLp % 400) == 0) ||
         ( ((DTLp % 4) == 0) && ((DTLp % 100) != 0) ) )
      DaTotal++;
    DTLp++;
  }
  // Add total basic days in prior years.
  DaTotal = DaTotal + ( (InDate[YEAR_ITEM] - 1997) * 365 );
  DTLp = 1;
  // Add days for elapsed months in current year.
  while ( DTLp < InDate[MONTH_ITEM] )
  {
    DaTotal = DaTotal + DaysPerMonth[ DTLp - 1 ];
    DTLp++;
  }
  // Add elapsed days in current month.
  DaTotal += InDate[DAY_ITEM];
  // Return total hours.
  return( (DaTotal * 24) + InTime[HOUR_ITEM] );
}


// Return day of week index 0-6 given date int array.
long DOW( int InDate[DATE_ITEMS] )
{
  int NoTime[TIME_ITEMS];
  NoTime[HOUR_ITEM] = 0;
  NoTime[MINUTE_ITEM] = 0;
  NoTime[SECOND_ITEM] = 0;
  return( ((DTHour(InDate, NoTime) / 24) + 2) % 7 );
}


// Parse verbose date (Monday, 1/19/1997) to integer date array.
int parse_date(char *In_Date, int Out_Date[DATE_ITEMS])
{
  int i;
  for (i=0; i<DATE_ITEMS; i++) Out_Date[i] = -1;
  // Find first digit.
  for (i=0; ( (i<RES_DATE_LEN_MAX) && (! isdigit(In_Date[i])) ); i++);
  if (isdigit(In_Date[i]))                   // Month first digit.
    Out_Date[MONTH_ITEM] = (In_Date[i++]-'0');
  else
    return FAIL;
  if (isdigit(In_Date[i]))                   // Optional month second digit.
    Out_Date[MONTH_ITEM] = ((Out_Date[MONTH_ITEM]*10) + (In_Date[i++]-'0'));
  i++;  // Skip slash.
  if (isdigit(In_Date[i]))                   // Day first digit.
    Out_Date[DAY_ITEM] = (In_Date[i++]-'0');
  else
    return FAIL;
  if (isdigit(In_Date[i]))                   // Optional day second digit.
    Out_Date[DAY_ITEM] = ((Out_Date[DAY_ITEM]*10) + (In_Date[i++]-'0'));
  i++;  // Skip slash.
  if ( ((i+3) >= RES_DATE_LEN_MAX) || ((i+3) >= strlen(In_Date)) )
    return FAIL;
  if ( (isdigit(In_Date[i])) && (isdigit(In_Date[i+1])) &&
       (isdigit(In_Date[i+2])) && (isdigit(In_Date[i+3])) )
  {
    Out_Date[YEAR_ITEM] = ( ((In_Date[i]-'0') * 1000) +
                            ((In_Date[i+1]-'0') * 100) +
                            ((In_Date[i+2]-'0') * 10) +
                            (In_Date[i+3]-'0') );
    return PASS;
  }
  else
    return FAIL;
}


// Convert YYYYMMDD to integer array dates.
int convert_date(char *In_Date, int Out_Date[DATE_ITEMS])
{
  Out_Date[YEAR_ITEM] = -1;
  Out_Date[MONTH_ITEM] = -1;
  Out_Date[DAY_ITEM] = -1;
  if (strlen(In_Date) != 8) return FAIL;
  Out_Date[YEAR_ITEM] = ( (1000 * (In_Date[0] - '0')) +
                          (100 * (In_Date[1] - '0')) +
                          (10 * (In_Date[2] - '0')) +
                          (In_Date[3] - '0') );
  Out_Date[MONTH_ITEM] = ( (10 * (In_Date[4] - '0')) +
                           (In_Date[5] - '0') );
  Out_Date[DAY_ITEM] = ( (10 * (In_Date[6] - '0')) +
                         (In_Date[7] - '0') );
  return PASS;
}


// Return date neatly formated.
int ShowDate( int InDate[DATE_ITEMS], char *OutStr )
{
  char YrStr[15], MoStr[15], DOWStr[15], DaStr[15];

  strcpy( OutStr, "invalid date" );
  if ( ! ValidYMD( InDate ) )
    return( -1.0 );
  sprintf( YrStr, "%d", InDate[YEAR_ITEM] );
  strcpy( DOWStr, DayName[ DOW(InDate) ] );
  strcpy( MoStr, MonthName[ InDate[MONTH_ITEM] - 1 ] );
  sprintf( DaStr, "%d", InDate[DAY_ITEM] );
  if ((InDate[DAY_ITEM] < 4) || (InDate[DAY_ITEM] > 20))
    InDate[DAY_ITEM] = (InDate[DAY_ITEM] % 10);
/*  if ( InDate[DAY_ITEM] == 1 )
    strcat( DaStr, "st." );
  else
  if ( InDate[DAY_ITEM] == 2 )
    strcat( DaStr, "nd." );
  else
  if ( InDate[DAY_ITEM] == 3 )
    strcat( DaStr, "rd." );
  else
    strcat( DaStr, "th." ); */
  sprintf( OutStr, "%s, %s %s, %s", DOWStr, MoStr, DaStr, YrStr );
  return(TRUE);
}


// Return time neatly formatted.
int ShowTime( int InTime[TIME_ITEMS], char *OutStr )
{
  char HrStr[3], MnStr[3], APStr[3];
  if (InTime[HOUR_ITEM] == 0)
  {
    sprintf( HrStr, "12" );
    sprintf( APStr, "am" );
  }
  else
    if (InTime[HOUR_ITEM] < 12)
    {
      sprintf( HrStr, "%d", InTime[HOUR_ITEM] );
      sprintf( APStr, "am" );
    }
    else
    {
      sprintf( HrStr, "%d", (InTime[HOUR_ITEM] - 12) );
      sprintf( APStr, "pm" );
    }
  if (InTime[MINUTE_ITEM] < 10)
    sprintf( MnStr, "0%d", InTime[MINUTE_ITEM] );
  else
    sprintf( MnStr, "%d", InTime[MINUTE_ITEM] );
  sprintf( OutStr, "%s:%s %s", HrStr, MnStr, APStr );
  return(TRUE);
}


// Return zero padded two digit integer.
char *twochar( int InNum )
{
  static char twochar_out[3];

  InNum = InNum % 100;
  if ( InNum < 10 )
    sprintf( twochar_out, "0%d", InNum );
  else
    sprintf( twochar_out, "%d", InNum );
  return( twochar_out );
}

