/* Single keystroke fetch routine.
   Put the big procedure here in any program requiring single
   keypress handling, not a simple task in C especially under Unix.
   The getkey function takes a timeout argument.
   Timeout >0 waits specified seconds for a keypress.
   Timeout 0 return EOF immediately if no keypresses buffered.
   Timeout <0 waits forever.
   Andrew Bedno - 1996.07.31  */

#include <stdio.h>
#include <termio.h>

int getkey(int getkey_timeout)
{
  int inch;
  int sttyerr;
  int stdin_fnum;
  int timearg, minarg;
  int timeloop;
  struct termio newsttyrec;
  struct termio oldsttyrec;
  /* Set stty raw... */
  stdin_fnum = fileno( stdin );
  /* Preserve current settings. */
  sttyerr = ioctl( stdin_fnum, TCGETA, &oldsttyrec );
  /* Copy current settings to working new record. */
  sttyerr = ioctl( stdin_fnum, TCGETA, &newsttyrec );
  /* Set input modes:  (Extra-paranoid, but it works.) */
  newsttyrec.c_iflag &= ~(IGNBRK);   // Ignore break.
  newsttyrec.c_iflag &= ~(BRKINT);   // Do not send interrupt on break.
  newsttyrec.c_iflag |= (IGNPAR);    // Ignore parity/framing errors.
  newsttyrec.c_iflag |= (PARMRK);    // Use char 0 on errors.
  newsttyrec.c_iflag &= ~(INPCK);    // Do not check parity.
  newsttyrec.c_iflag &= ~(ISTRIP);   // Do not strip top bit.
  newsttyrec.c_iflag &= ~(INLCR);    // Do not convert NL/CR.
  newsttyrec.c_iflag &= ~(IGNCR);    // Do not ignore CR.
  newsttyrec.c_iflag &= ~(ICRNL);    // Do not ignore NL.
  newsttyrec.c_iflag &= ~(IUCLC);    // Do not map to lower case.
  newsttyrec.c_iflag &= ~(IXON);     // Do not use XON flow control.
  newsttyrec.c_iflag &= ~(IXANY);    // Do not use XANY flow control.
  newsttyrec.c_iflag &= ~(IXOFF);    // Do not use XOFF flow control.
  newsttyrec.c_iflag &= ~(IMAXBEL);  // Do not ding on buffer full.
  /* Set output modes: */
  newsttyrec.c_oflag &= ~(OPOST);    // Do not post process output.
  newsttyrec.c_oflag &= ~(OLCUC);    // Do not map to upper case.
  newsttyrec.c_oflag &= ~(ONLCR);    // Do not convert NL/CR.
  newsttyrec.c_oflag &= ~(OCRNL);    // Do not convert CR/NL.
  newsttyrec.c_oflag &= ~(ONOCR);    // No auto CR at col 0.
  newsttyrec.c_oflag &= ~(ONLRET);   // Do not output CR with NL.
  newsttyrec.c_oflag &= ~(OFILL);    // Do not send fill chars.
  newsttyrec.c_oflag &= ~(OFDEL);    // Fill is char 0.
  newsttyrec.c_oflag &= ~(NLDLY);
  newsttyrec.c_oflag |= (NL0);       // No NL delay (NLDLY).
  newsttyrec.c_oflag &= ~(CRDLY);
  newsttyrec.c_oflag |= (CR0);       // No CR delay (CRDLY).
  newsttyrec.c_oflag &= ~(TABDLY);
  newsttyrec.c_oflag |= (TAB0);      // No TAB delay (TABDLY).
  newsttyrec.c_oflag &= ~(BSDLY);
  newsttyrec.c_oflag |= (BS0);       // No BS delay (BSDLY).
  newsttyrec.c_oflag &= ~(VTDLY);
  newsttyrec.c_oflag |= (VT0);       // No VT delay (VTDLY).
  newsttyrec.c_oflag &= ~(FFDLY);
  newsttyrec.c_oflag |= (FF0);       // No FF delay (FFDLY).
  /* Set control modes: */
  newsttyrec.c_cflag &= ~(CSIZE);
  newsttyrec.c_cflag |= (CS8);       // Eight data bits (CSIZE).
  newsttyrec.c_cflag &= ~(CSTOPB);   // No stop bits.
  newsttyrec.c_cflag |= (CREAD);     // Enable reading.
  newsttyrec.c_cflag &= ~(PARENB);   // Parity not enabled.
  newsttyrec.c_cflag &= ~(PARODD);   // Parity not odd.
  newsttyrec.c_cflag |= (HUPCL);     // Hangup on exit.
  newsttyrec.c_cflag &= ~(CLOCAL);   // Do not ignore modem control.
  newsttyrec.c_cflag |= (CRTSCTS);   // Use hardware handshaking.
  /* Set local modes: */
  newsttyrec.c_lflag &= ~(ISIG);     // Disable signals.
  newsttyrec.c_lflag &= ~(ICANON);   // Disable input processing.
  newsttyrec.c_lflag &= ~(XCASE);    // Do not change case.
  newsttyrec.c_lflag &= ~(ECHO);     // Do not echo input.
  newsttyrec.c_lflag &= ~(ECHOE);    // Do not echo erase char.
  newsttyrec.c_lflag &= ~(ECHOK);    // Do not echo kill char.
  newsttyrec.c_lflag &= ~(ECHONL);   // Do not echo NL.
  newsttyrec.c_lflag &= ~(ECHOCTL);  // Do not echo control chars.
  newsttyrec.c_lflag &= ~(ECHOPRT);  // Do not print on erase.
  newsttyrec.c_lflag &= ~(ECHOKE);   // Do not print on kill.
  newsttyrec.c_lflag &= ~(FLUSHO);   // Do not flush now.
  newsttyrec.c_lflag &= ~(NOFLSH);   // Do not flush on signal.
  newsttyrec.c_lflag &= ~(TOSTOP);   // Do not signal to back group.
  newsttyrec.c_lflag &= ~(PENDIN);   // Do not buffer typeahead.
  newsttyrec.c_lflag &= ~(IEXTEN);   // Disable extended processing.
  /* Set timing. */
  if (getkey_timeout == 0)
  {
    timearg = 0;
    minarg = 0;
  }
  if (getkey_timeout < 0)
  {
    timearg = 0;
    minarg = 1;
  }
  if (getkey_timeout > 0)
  {
    timearg = getkey_timeout;
    minarg = 0;
  }
  /* Activate revised port settings. */
  newsttyrec.c_cc[VMIN] = minarg;
  if (timearg > 0)
    newsttyrec.c_cc[VTIME] = 10;
  else
    newsttyrec.c_cc[VTIME] = 0;
  sttyerr = ioctl( stdin_fnum, TCSETA, &newsttyrec );
  // Get the key.  OS handles a short timeout.
  timeloop = 0;
  do
  {
    if (timearg > 0) timeloop++;
    inch = fgetc(stdin);
  }
  while ( (inch == EOF) && ((timeloop < timearg) || (timearg < 0)) );
  /* Restore old settings. */
  sttyerr = ioctl( stdin_fnum, TCSETA, &oldsttyrec );
  return( inch );
}


/* Sample usage:
main()
{
  char ch;
  ch = getkey(-1);
  fprintf( stdout, "%c", ch );
}
*/

