/*************************************************************************/
/*************************************************************************/
/******                                                             ******/
/******  STAND ALONE, PORTABLE, TWINKLING CHRISTMAS TREE            ******/
/******                                                             ******/
/*************************************************************************/
/*************************************************************************/

#include <stdio.h>
#include <ctype.h>
#include <termio.h>

/*-----------------------------------------------------------------------*/
/* <vid_init> returns true if tput operates (TERM variable set).         */
/*-----------------------------------------------------------------------*/

static char vid_clear_seq[40] = "";   /* clear screen */
static char vid_civis_seq[40] = "";   /* cursor invisible */
static char vid_cnorm_seq[40] = "";   /* cursor normal */
static char vid_rev_seq[40] = "";     /* start reverse attribute */
static char vid_blink_seq[40] = "";   /* start blink attribute */
static char vid_bold_seq[40] = "";    /* start bright attribute */
static char vid_dim_seq[40] = "";     /* start dim attribute */
static char vid_ul_seq[40] = "";      /* start underline attribute */
static char vid_norm_seq[40] = "";    /* reset all attributes */

int vid_ok = 0;                /* flags successful init */

/* <vid_read_tput> reads from tput, used by vid_init. */

int vid_read_tput( vid_seq, vid_cmd )
char *vid_seq, *vid_cmd;
{
  FILE *vid_pipe;
  int vid_read_size;
  char vid_cmd_out[50];

  *vid_seq = '\0';
  sprintf( vid_cmd_out, "tput %s 2>/dev/null", vid_cmd );
  vid_pipe = popen( vid_cmd_out, "r" );
  if ( vid_pipe != NULL )
  {
    vid_read_size = fread( vid_seq, 1, 40, vid_pipe );
    if ( vid_read_size > 0 )
      vid_seq[ vid_read_size ] = '\0';
    else
      vid_seq[ 0 ] = '\0';
    pclose( vid_pipe );
  }
  return( strlen( vid_seq ) );
}

int vid_init()
{
  int vid_init_out = 0;

  vid_init_out = vid_read_tput( vid_clear_seq, "clear" ) || vid_init_out;
  vid_init_out = vid_read_tput( vid_civis_seq, "civis" ) || vid_init_out;
  vid_init_out = vid_read_tput( vid_cnorm_seq, "cnorm" ) || vid_init_out;
  vid_init_out = vid_read_tput( vid_rev_seq, "rev" ) || vid_init_out;
  vid_init_out = vid_read_tput( vid_blink_seq, "blink" ) || vid_init_out;
  vid_init_out = vid_read_tput( vid_bold_seq, "bold" ) || vid_init_out;
  vid_init_out = vid_read_tput( vid_dim_seq, "dim" ) || vid_init_out;
  vid_init_out = vid_read_tput( vid_ul_seq, "smul" ) || vid_init_out;
  vid_init_out = vid_read_tput( vid_norm_seq, "sgr0" ) || vid_init_out;
  vid_ok = vid_init_out == 1;
  return( vid_init_out );
}
  
/*-----------------------------------------------------------------------*/
/* <vid_clear> clears the screen.                                        */
/*-----------------------------------------------------------------------*/

int vid_clear()
{
  printf( vid_clear_seq );
  fflush( stdout );
  return( 0 );
}

/*-----------------------------------------------------------------------*/
/* <vid_gotoXY> moves the cursor to a selected location.                 */
/*-----------------------------------------------------------------------*/

int vid_gotoXY( vid_x, vid_y )
int vid_x, vid_y;
{
  char vid_cmd[50];

  fflush( stdout );
  sprintf( vid_cmd, "tput cup %d %d 2>/dev/null", vid_y, vid_x );
  return( system( vid_cmd ) );
}

/*-----------------------------------------------------------------------*/
/* <vid_attr> sets selected video attributes on terminal.                */
/*-----------------------------------------------------------------------*/

#define v_norm 0
#define v_rev 1
#define v_blink 2
#define v_bold 4
#define v_dim 8
#define v_ul 16

int vid_attr( vid_attr_val )
int vid_attr_val;
{
  char vid_attr_seq[160];

  strcpy( vid_attr_seq, vid_norm_seq );
  if ( vid_attr_val & v_rev ) strcat( vid_attr_seq, vid_rev_seq );
  if ( vid_attr_val & v_blink ) strcat( vid_attr_seq, vid_blink_seq );
  if ( vid_attr_val & v_bold ) strcat( vid_attr_seq, vid_bold_seq );
  if ( vid_attr_val & v_dim ) strcat( vid_attr_seq, vid_dim_seq );
  if ( vid_attr_val & v_ul ) strcat( vid_attr_seq, vid_ul_seq );
  printf( vid_attr_seq );
  fflush( stdout );
  return( 0 );
}

/*-----------------------------------------------------------------------*/
/* <vid_cursor> sets selected cursor mode.                               */
/*-----------------------------------------------------------------------*/

#define v_con 0
#define v_coff 1

int vid_cursor( vid_cstyle )
int vid_cstyle;
{
  if ( vid_cstyle == v_coff )
    printf( vid_civis_seq );
  else
    printf( vid_cnorm_seq );
  fflush( stdout );
  return( 0 );
}

/*-----------------------------------------------------------------------*/
/* <vid_print> prints text, including attributes, at a location.         */
/*-----------------------------------------------------------------------*/

int vid_print( vid_x, vid_y, vid_attr_in, vid_txt )
int vid_x, vid_y, vid_attr_in;
char *vid_txt;
{
  vid_gotoXY( vid_x, vid_y );
  vid_attr( vid_attr_in );
  printf( vid_txt );
  fflush( stdout );
  return( 0 ); 
}

/*-----------------------------------------------------------------------*/
/* <memmove> is a safe memcpy, s2 to s1, which allows overlapping moves. */
/* If regions overlap s2 will be destroyed.  ANSI but missing from AT&T. */ 
/*-----------------------------------------------------------------------*/

char *memmove( s1, s2, n )
char *s1, *s2;
int n;
{
  char *S1, *S2, *S3;

  if ( n && s1 && s2 )  /* check for size and nulls */
  {
    if ( s2 > s1 )
    {  /* ascending copy */
      S1 = s1;  S2 = s2;  S3 = s2 + n;
      do
      {
        *S1++ = *S2++;
      } while ( S2 != S3 );
    }
    else    
    {  /* descending copy */
      S1 = s1 + n - 1;  S2 = s2 + n - 1;  S3 = s2 - 1;
      do
      {
        *S1-- = *S2--;
      } while ( S2 != S3 );
    }      
  }
  return( s1 );
}

/*-----------------------------------------------------------------------*/
/* <stty_raw> <stty_cooked> These procedures change the stty settings    */
/* for stdin to disable or re-enable buffering and echo.                 */
/* They are primarily for use by ReadKey and kbhit routines.             */
/* Can be used to globally disable echo, or re-enable on termination.    */
/*-----------------------------------------------------------------------*/

static struct termio sttyold, sttynew;  /* cooked and raw stty settings */
/* fgetc returns NOKEY if no key pressed and stdin is unbuffered */
#define NOKEY -1
/* PrevKey preserves the keystroke eaten by kbhit */
static char PrevKey = NOKEY;
static int stty_is_raw = 0;  /* current state */

int stty_raw()
{
  int sttyerr = 0;
  int stdin_fnum;

  if ( ! stty_is_raw )
  {
    /* get current settings */
    stdin_fnum = fileno( stdin );
    sttyerr = ioctl( stdin_fnum, TCGETA, &sttyold );
    memmove( &sttynew, &sttyold, sizeof( sttyold ) );
    /* turn off bits to disable line buffering and echo */
    sttynew.c_lflag =
      sttyold.c_lflag & ~( ICANON | ECHO | ECHOE | ECHONL );
    /* clear values to disable character buffering and timeout */
    sttynew.c_cc[ VMIN ] = 0;  sttynew.c_cc[ VTIME ] = 0;
    /* engage new settings */
    if ( ! sttyerr )
      sttyerr = ioctl( stdin_fnum, TCSETA, &sttynew );
    stty_is_raw = ! sttyerr;
    return( sttyerr );
  }
  return( 0 );
}

int stty_cooked()
{
  int sttyerr = 0;
  int stty_mask, stdin_fnum;

  if ( stty_is_raw )
  {
    /* get current settings */
    stdin_fnum = fileno( stdin );
    sttyerr = ioctl( stdin_fnum, TCGETA, &sttynew );
    /* get original values of cleared bits */
    stty_mask =
      sttyold.c_lflag & ( ICANON | ECHO | ECHOE | ECHONL );
    /* clear those bits from current */
    sttynew.c_lflag =
      sttynew.c_lflag & ~( ICANON | ECHO | ECHOE | ECHONL );
    /* set those bits to original values */
    sttynew.c_lflag = sttynew.c_lflag | stty_mask;
    /* set buffering and timeout values to original */
    sttynew.c_cc[ VMIN ] = sttyold.c_cc[ VMIN ];
    sttynew.c_cc[ VTIME ] = sttyold.c_cc[ VTIME ];
    /* engage new settings */
    if ( ! sttyerr )
      sttyerr = ioctl( stdin_fnum, TCSETA, &sttynew );
    stty_is_raw = sttyerr;
    return( sttyerr );
  }
  return( 0 );
}

/*-----------------------------------------------------------------------*/
/* <ReadKey> reads one keypress from stdin and error checks.             */
/*-----------------------------------------------------------------------*/

/* These attributes are set or reset only by the application. */
int READKEY_NOECHO = 0;  /* extern */  /* don't echo keypress */
int READKEY_UC = 0;  /* extern */      /* translate to upper case */

int ReadKey()
{
  int ReadKeyOut;
  int stty_was_raw;

  if ( PrevKey != NOKEY )
  {
    ReadKeyOut = PrevKey;  /* preserved from kbhit() */
    PrevKey = NOKEY;
  }
  else
  {
    stty_was_raw = stty_is_raw;
    stty_raw();
    do
      ReadKeyOut = fgetc( stdin );
    while ( ReadKeyOut == NOKEY );
    if ( ! stty_was_raw )  /* continues prior raw mode */
      stty_cooked();
    if ( ! READKEY_NOECHO && isprint( ReadKeyOut ) )
      putchar( ReadKeyOut );
  }
  if ( READKEY_UC )
    ReadKeyOut = toupper( ReadKeyOut );
  return( ReadKeyOut );
}

/*-----------------------------------------------------------------------*/
/* <kbhit> returns true if a character is waiting in stdin.  Note that   */ 
/* one kbhit uses a destructive technique, so next character must be     */
/* read using ReadKey, not fget* or get*.  ANSI but missing from AT&T.   */
/*-----------------------------------------------------------------------*/

int kbhit()
{
  int KeyPressedOut;
  int stty_was_raw;

  if ( PrevKey != NOKEY )
    KeyPressedOut = PrevKey;
  else
  {
    stty_was_raw = stty_is_raw;
    stty_raw();
    KeyPressedOut = fgetc( stdin );
    if ( KeyPressedOut != NOKEY )
      PrevKey = KeyPressedOut;
    if ( ! stty_was_raw )  /* continues prior raw mode */
      stty_cooked();
  }
  return( KeyPressedOut != NOKEY );
}

/*********************************************************************/

int main()
{
  typedef struct light_rec
  {
    int x, y, lit;
  }
  light_rec;
#define MaxLight 200
  struct light_rec light[ MaxLight ];
  int lights = -1;
  int z, y, quit = 0;
  int x = 40, wid = 1, start = 1, height = 19, size = 1, light_gap = 5;
  char p_ln[ 80 ];
  
  if ( vid_init() )
  {
    vid_cursor( v_coff );
    vid_clear();
    /* draw a tree */
    for ( y = start; ( ( y <= height ) || ( x < 1 ) ); y++ ) 
    {
      memset( p_ln, '*', wid );  p_ln[wid] = '\0';
      vid_print( x, y, v_norm, p_ln );
      /* put some lights on this row */
      z = x + 1;
      while ( ( lights < MaxLight - 1 ) && ( z < x + wid - 1 ) )
      {
        z = z + ( rand() % light_gap );
        if ( z < x + wid - 1 )
        {
          lights++;
          light[ lights ].x = z; 
          light[ lights ].y = y; 
          light[ lights ].lit = rand() % 2; 
        }
        z += 4;
      }
      x = x - size;  wid = wid + ( size * 2 );
    }
    /* draw the trunk */
    vid_print( 37, y++, v_norm, "XXXXXXX" );
    vid_print( 37, y++, v_norm, "XXXXXXX" );
    vid_print( 37, y++, v_norm, "XXXXXXX" );
    /* put the light on top */
    lights++;
    light[ lights ].x = 40;
    light[ lights ].y = 1;
    light[ lights ].lit = rand() % 2;
    /* draw the lights */
    for ( z = 0; z <= lights; z++ )
    {
      if ( light[ z ].lit )
        vid_print( light[ z ].x, light[ z ].y, v_rev, "O" );
      else
        vid_print( light[ z ].x, light[ z ].y, v_norm, "O" );
    }
    vid_print( 1, 24, v_dim, "Press  <ESC>  to stop twinkling." );
    stty_raw();
    /* twinkle */
    do
    {
      z = rand() % ( lights + 1 );
      if ( light[ z ].lit )
        vid_print( light[ z ].x, light[ z ].y, v_rev, "O" );
      else
        vid_print( light[ z ].x, light[ z ].y, v_norm, "O" );
      light[ z ].lit = ! light[ z ].lit;
      if ( kbhit() )
        quit = ReadKey() == 27;
    } while ( ! quit );
    vid_cursor( v_con );
    stty_cooked();
    vid_attr( v_norm );
    vid_clear();
    printf( "xmastree: Happy Holidays\n\n" );
  }

  return( 0 );
}
